@@ -15,20 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include "relaylib.h"
-#include "cbuf2.h"
-#include "debug.h"
-#include "ripogg.h"
-
-/*****************************************************************************
- * Global variables
- *****************************************************************************/
-/* The circular buffer */
-CBUF2 g_cbuf2;
-
 /*****************************************************************************
  * Notes:
  *
@@ -45,7 +31,21 @@ CBUF2 g_cbuf2;
  *   The read & write indices always point to the next byte to be read 
  *   or written.  When the buffer is full or empty, they are equal.
  *   Use the item_count to distinguish these cases.
+ *
+ *   cbuf2->base_idx refers to the beginning of the filled region
+ *
+ *   Chunks are always absolute
+ *
  *****************************************************************************/
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include "errors.h"
+#include "relaylib.h"
+#include "cbuf2.h"
+#include "debug.h"
+#include "ripogg.h"
+
 /*****************************************************************************
  * Private functions
  *****************************************************************************/
@@ -53,14 +53,12 @@ static u_long cbuf2_idx_to_chunk (CBUF2 *cbuf2, u_long idx);
 static u_long cbuf2_add (CBUF2 *cbuf2, u_long pos, u_long len);
 static void cbuf2_get_used_fragments (CBUF2 *cbuf2, u_long* frag1, 
 				      u_long* frag2);
-static void cbuf2_advance_metadata_list (CBUF2* cbuf2);
-static error_code cbuf2_extract_relay_mp3 (CBUF2 *cbuf2, char *data, 
-					   u_long *pos, u_long *len, 
-					   int icy_metadata);
+static void cbuf2_advance_metadata_list (RIP_MANAGER_INFO* rmi, CBUF2* cbuf2);
+static error_code cbuf2_extract_relay_mp3 (CBUF2 *cbuf2, RELAY_LIST* ptr);
 static error_code cbuf2_extract_relay_ogg (CBUF2 *cbuf2, RELAY_LIST* ptr);
 static u_long cbuf2_offset (CBUF2 *cbuf2, u_long pos);
 static u_long cbuf2_offset_2 (CBUF2 *cbuf2, u_long pos);
-static u_long cbuf2_subtract (CBUF2 *cbuf2, u_long pos1, u_long pos2);
+static void cbuf2_debug_lists (RIP_MANAGER_INFO* rmi, CBUF2 *cbuf2);
 
 /*****************************************************************************
  * Function definitions
@@ -127,7 +125,7 @@ cbuf2_insert (CBUF2 *cbuf2, const char *data, u_long count)
 }
 
 static void
-cbuf2_advance_metadata_list (CBUF2* cbuf2)
+cbuf2_advance_metadata_list (RIP_MANAGER_INFO* rmi, CBUF2* cbuf2)
 {
     u_long frag1, frag2;
     u_long metadata_pos;
@@ -166,15 +164,15 @@ cbuf2_advance_metadata_list (CBUF2* cbuf2)
 	}
     }
     debug_printf ("CBUF_EXTRACT\n");
-    cbuf2_debug_lists (cbuf2);
+    cbuf2_debug_lists (rmi, cbuf2);
 }
 
 static void
-cbuf2_advance_relay_list (CBUF2* cbuf2, u_long count)
+cbuf2_advance_relay_list (RIP_MANAGER_INFO* rmi, CBUF2* cbuf2, u_long count)
 {
     RELAY_LIST *prev, *ptr, *next;
 
-    ptr = g_relay_list;
+    ptr = rmi->relay_list;
     if (ptr == NULL) {
 	return;
     }
@@ -182,16 +180,16 @@ cbuf2_advance_relay_list (CBUF2* cbuf2, u_long count)
     while (ptr != NULL) {
 	next = ptr->m_next;
 	if (!ptr->m_is_new) {
-	    if (ptr->m_cbuf_pos >= count) {
-		u_long old_pos = ptr->m_cbuf_pos;
-		ptr->m_cbuf_pos -= count;
+	    if (ptr->m_cbuf_offset >= count) {
+		u_long old_pos = ptr->m_cbuf_offset;
+		ptr->m_cbuf_offset -= count;
 		debug_printf ("Updated relay pointer: %d -> %d\n",
-			      old_pos, ptr->m_cbuf_pos);
+			      old_pos, ptr->m_cbuf_offset);
 		prev = ptr;
 	    } else {
 		debug_printf ("Relay: Client %d couldn't keep up with cbuf\n", 
 			      ptr->m_sock);
-		relaylib_disconnect (prev, ptr);
+		relaylib_disconnect (rmi, prev, ptr);
 	    }
 	}
 	ptr = next;
@@ -206,7 +204,7 @@ cbuf2_set_next_song (CBUF2 *cbuf2, u_long pos)
 
 /* Advance the cbuffer while respecting ogg page boundaries */
 error_code
-cbuf2_advance_ogg (CBUF2 *cbuf2, int requested_free_size)
+cbuf2_advance_ogg (RIP_MANAGER_INFO* rmi, CBUF2 *cbuf2, int requested_free_size)
 {
     u_long count;
     LIST *p, *n;
@@ -217,8 +215,8 @@ cbuf2_advance_ogg (CBUF2 *cbuf2, int requested_free_size)
     }
 
     if (cbuf2->have_relay) {
-	debug_printf ("Waiting for g_relay_list_sem\n");
-	threadlib_waitfor_sem (&g_relay_list_sem);
+	debug_printf ("Waiting for rmi->relay_list_sem\n");
+	threadlib_waitfor_sem (&rmi->relay_list_sem);
     }
     debug_printf ("Waiting for cbuf2->cbuf_sem\n");
     threadlib_waitfor_sem (&cbuf2->cbuf_sem);
@@ -249,11 +247,11 @@ cbuf2_advance_ogg (CBUF2 *cbuf2, int requested_free_size)
     cbuf2->item_count -= count;
 
     if (cbuf2->have_relay) {
-	cbuf2_advance_relay_list (cbuf2, count);
+	cbuf2_advance_relay_list (rmi, cbuf2, count);
     }
     threadlib_signal_sem (&cbuf2->cbuf_sem);
     if (cbuf2->have_relay) {
-	threadlib_signal_sem (&g_relay_list_sem);
+	threadlib_signal_sem (&rmi->relay_list_sem);
     }
 
     if (cbuf2->item_count + requested_free_size <= cbuf2->size) {
@@ -266,7 +264,8 @@ cbuf2_advance_ogg (CBUF2 *cbuf2, int requested_free_size)
 /* On extract, we need to update all the relay pointers. */
 /* GCS Now it always extracts a chunk */
 error_code
-cbuf2_extract (CBUF2 *cbuf2, char *data, u_long count, u_long* curr_song)
+cbuf2_extract (RIP_MANAGER_INFO* rmi, CBUF2 *cbuf2, 
+	       char *data, u_long count, u_long* curr_song)
 {
     u_long frag1, frag2;
 
@@ -276,8 +275,8 @@ cbuf2_extract (CBUF2 *cbuf2, char *data, u_long count, u_long* curr_song)
     }
 
     if (cbuf2->have_relay) {
-	debug_printf ("Waiting for g_relay_list_sem\n");
-	threadlib_waitfor_sem (&g_relay_list_sem);
+	debug_printf ("Waiting for rmi->relay_list_sem\n");
+	threadlib_waitfor_sem (&rmi->relay_list_sem);
     }
     debug_printf ("Waiting for cbuf2->cbuf_sem\n");
     threadlib_waitfor_sem (&cbuf2->cbuf_sem);
@@ -300,14 +299,14 @@ cbuf2_extract (CBUF2 *cbuf2, char *data, u_long count, u_long* curr_song)
 	cbuf2->next_song -= count;
     }
 
-    cbuf2_advance_metadata_list (cbuf2);
+    cbuf2_advance_metadata_list (rmi, cbuf2);
 
     if (cbuf2->have_relay) {
-	cbuf2_advance_relay_list (cbuf2, count);
+	cbuf2_advance_relay_list (rmi, cbuf2, count);
     }
     threadlib_signal_sem (&cbuf2->cbuf_sem);
     if (cbuf2->have_relay) {
-	threadlib_signal_sem (&g_relay_list_sem);
+	threadlib_signal_sem (&rmi->relay_list_sem);
     }
     return SR_SUCCESS;
 }
@@ -349,7 +348,8 @@ cbuf2_peek (CBUF2 *cbuf2, char *data, u_long count)
 }
 
 error_code
-cbuf2_insert_chunk (CBUF2 *cbuf2, const char *data, u_long count,
+cbuf2_insert_chunk (RIP_MANAGER_INFO* rmi,
+		    CBUF2 *cbuf2, const char *data, u_long count,
 		    int content_type, TRACK_INFO* ti)
 {
     LIST this_page_list;
@@ -364,7 +364,7 @@ cbuf2_insert_chunk (CBUF2 *cbuf2, const char *data, u_long count,
 	LIST *p;
 	unsigned long page_loc;
 
-	rip_ogg_process_chunk (&this_page_list, data, count, ti);
+	rip_ogg_process_chunk (rmi, &this_page_list, data, count, ti);
 
 	if (list_empty(&cbuf2->ogg_page_list)) {
 	    page_loc = 0;
@@ -374,7 +374,7 @@ cbuf2_insert_chunk (CBUF2 *cbuf2, const char *data, u_long count,
 	    page_loc = cbuf2_add (cbuf2, tmp->m_page_start, tmp->m_page_len);
 	}
 	
-	p = &this_page_list.next;
+	// p = &this_page_list.next;
 	list_for_each (p, &this_page_list) {
 	    tmp = list_entry(p, OGG_PAGE_LIST, m_list);
 	    tmp->m_page_start = page_loc;
@@ -395,22 +395,28 @@ cbuf2_insert_chunk (CBUF2 *cbuf2, const char *data, u_long count,
 	list_splice (&this_page_list, cbuf2->ogg_page_list.prev);
     } else if (ti && ti->have_track_info) {
 	/* Insert metadata data */
+#if defined (commentout)
 	int num_bytes;
 	unsigned char num_16_bytes;
+#endif
 	METADATA_LIST* ml;
 	ml = (METADATA_LIST*) malloc (sizeof(METADATA_LIST));
 	/* GCS FIX: Check malloc error */
 	ml->m_chunk = chunk_no;
+#if defined (commentout)
 	num_bytes = snprintf (&ml->m_composed_metadata[1], MAX_METADATA_LEN,
 			      "StreamTitle='%s - %s';", 
 			      ti->artist, ti->title);
 	ml->m_composed_metadata[MAX_METADATA_LEN] = 0;  // note, not LEN-1
 	num_16_bytes = (num_bytes + 15) / 16;
 	ml->m_composed_metadata[0] = num_16_bytes;
+#endif
+	memcpy (ml->m_composed_metadata, ti->composed_metadata, 
+		MAX_METADATA_LEN+1);
 	list_add_tail (&(ml->m_list), &(cbuf2->metadata_list));
     }
     debug_printf ("CBUF_INSERT\n");
-    cbuf2_debug_lists (cbuf2);
+    cbuf2_debug_lists (rmi, cbuf2);
 
     threadlib_signal_sem (&cbuf2->cbuf_sem);
     return SR_SUCCESS;
@@ -425,9 +431,7 @@ cbuf2_extract_relay (CBUF2 *cbuf2, RELAY_LIST* ptr)
     if (cbuf2->content_type == CONTENT_TYPE_OGG) {
 	return cbuf2_extract_relay_ogg (cbuf2, ptr);
     } else {
-	return cbuf2_extract_relay_mp3 (cbuf2, ptr->m_buffer, 
-					&ptr->m_cbuf_pos, &ptr->m_left_to_send,
-					ptr->m_icy_metadata);
+	return cbuf2_extract_relay_mp3 (cbuf2, ptr);
     }
 }
 
@@ -456,16 +460,16 @@ cbuf2_extract_relay_ogg (CBUF2 *cbuf2, RELAY_LIST* ptr)
 	u_long relay_idx, remaining;
 
 	threadlib_waitfor_sem (&cbuf2->cbuf_sem);
-	relay_idx = cbuf2_add (cbuf2, cbuf2->base_idx, ptr->m_cbuf_pos);
+	relay_idx = cbuf2_add (cbuf2, cbuf2->base_idx, ptr->m_cbuf_offset);
 	cbuf2_get_used_fragments (cbuf2, &frag1, &frag2);
-	if (ptr->m_cbuf_pos < frag1) {
-	    remaining = frag1 - ptr->m_cbuf_pos;
+	if (ptr->m_cbuf_offset < frag1) {
+	    remaining = frag1 - ptr->m_cbuf_offset;
 	} else {
-	    remaining = cbuf2->item_count - ptr->m_cbuf_pos;
+	    remaining = cbuf2->item_count - ptr->m_cbuf_offset;
 	}
 	debug_printf ("%p p=%d/t=%d (f1=%d,f2=%d) [r=%d/bs=%d]\n", 
 		      ptr,
-		      ptr->m_cbuf_pos, cbuf2->item_count,
+		      ptr->m_cbuf_offset, cbuf2->item_count,
 		      frag1, frag2,
 		      remaining, ptr->m_buffer_size);
 	if (remaining == 0) {
@@ -475,11 +479,11 @@ cbuf2_extract_relay_ogg (CBUF2 *cbuf2, RELAY_LIST* ptr)
 	if (remaining > ptr->m_buffer_size) {
 	    memcpy (ptr->m_buffer, &cbuf2->buf[relay_idx], ptr->m_buffer_size);
 	    ptr->m_left_to_send = ptr->m_buffer_size;
-	    ptr->m_cbuf_pos += ptr->m_buffer_size;
+	    ptr->m_cbuf_offset += ptr->m_buffer_size;
 	} else {
 	    memcpy (ptr->m_buffer, &cbuf2->buf[relay_idx], remaining);
 	    ptr->m_left_to_send = remaining;
-	    ptr->m_cbuf_pos += remaining;
+	    ptr->m_cbuf_offset += remaining;
 	}
 	threadlib_signal_sem (&cbuf2->cbuf_sem);
     }
@@ -487,46 +491,51 @@ cbuf2_extract_relay_ogg (CBUF2 *cbuf2, RELAY_LIST* ptr)
 }
 
 static error_code 
-cbuf2_extract_relay_mp3 (CBUF2 *cbuf2, char *data, u_long *pos, u_long *len,
-			 int icy_metadata)
+cbuf2_extract_relay_mp3 (CBUF2 *cbuf2, RELAY_LIST* ptr)
 {
     u_long frag1, frag2;
     u_long relay_chunk_no;
     u_long relay_idx;
-    u_long old_pos;
+    u_long old_offset;
+
+
+    char *data = ptr->m_buffer;
+    u_long *offset = &ptr->m_cbuf_offset;
+    u_long *len = &ptr->m_left_to_send;       /* Amount that relay lib 
+						 needs to send to client */
+    int icy_metadata = ptr->m_icy_metadata;
+
     *len = 0;
 
     /* Check if there is data to send to the relay */
     threadlib_waitfor_sem (&cbuf2->cbuf_sem);
     cbuf2_get_used_fragments (cbuf2, &frag1, &frag2);
-    if (*pos + cbuf2->chunk_size > cbuf2->item_count) {
+    if (*offset + cbuf2->chunk_size > cbuf2->item_count) {
 	threadlib_signal_sem (&cbuf2->cbuf_sem);
 	return SR_ERROR_BUFFER_EMPTY;
     }
 
     /* Otherwise, there is enough data */
-    /* The *pos is the offset from the filled part of the buffer */
-    old_pos = *pos;
-    relay_idx = cbuf2_add (cbuf2, cbuf2->base_idx, *pos);
-    relay_chunk_no = cbuf2_idx_to_chunk (cbuf2, *pos);
+    /* The *offset is the offset from the filled part of the buffer */
+    old_offset = *offset;
+    relay_idx = cbuf2_add (cbuf2, cbuf2->base_idx, *offset);
+    relay_chunk_no = cbuf2_idx_to_chunk (cbuf2, relay_idx);
     memcpy (data, &cbuf2->buf[relay_idx], cbuf2->chunk_size);
-    *pos += cbuf2->chunk_size;
+    *offset += cbuf2->chunk_size;
     *len = cbuf2->chunk_size;
-    debug_printf ("Updated relay pointer: %d -> %d (%d %d %d)\n",
-		  old_pos, *pos, cbuf2->base_idx,
-		  relay_idx, cbuf2_write_index(cbuf2));
-    debug_printf ("Partial request fulfilled: %d bytes (%d)\n", *len, 
-		  icy_metadata);
+    debug_printf ("RELAY_DEBUG Was: offset=%d, idx=%d, chunk=%d\n", old_offset,
+		  relay_idx, relay_chunk_no);
+    debug_printf ("RELAY_DEBUG Changed to: offset=%d\n", *offset);
     if (icy_metadata) {
 	int have_metadata = 0;
 	LIST *p = cbuf2->metadata_list.next;
 	METADATA_LIST *tmp;
 	list_for_each (p, &(cbuf2->metadata_list)) {
 	    tmp = list_entry(p, METADATA_LIST, m_list);
-	    debug_printf ("Comparing metadata: meta=%d relay=%d\n",
+	    debug_printf ("RELAY_DEBUG Comparing metadata: meta=%d relay=%d\n",
 			  tmp->m_chunk, relay_chunk_no);
 	    if (tmp->m_chunk == relay_chunk_no) {
-		debug_printf ("Got metadata!\n");
+		debug_printf ("RELAY_DEBUG Got metadata!\n");
 		have_metadata = 1;
 		break;
 	    }
@@ -547,48 +556,51 @@ cbuf2_extract_relay_mp3 (CBUF2 *cbuf2, char *data, u_long *pos, u_long *len,
 }
 
 void
-cbuf2_debug_lists (CBUF2 *cbuf2)
+cbuf2_debug_lists (RIP_MANAGER_INFO* rmi, CBUF2 *cbuf2)
 {
     LIST *p;
 
-    debug_printf ("---CBUF_DEBUG_META_LIST---\n");
-    debug_printf("%ld: <size>\n", cbuf2->size);
-    debug_printf("%ld: Start\n", cbuf2->base_idx);
-    list_for_each (p, &(cbuf2->metadata_list)) {
-	METADATA_LIST *meta_tmp;
-	int metadata_pos;
-	meta_tmp = list_entry (p, METADATA_LIST, m_list);
-	metadata_pos = (meta_tmp->m_chunk + 1) * cbuf2->chunk_size - 1;
-	debug_printf("%ld: %d [%d]%s\n", 
-		     metadata_pos,
-		     meta_tmp->m_chunk, 
-		     (int) meta_tmp->m_composed_metadata[0],
-		     &meta_tmp->m_composed_metadata[1]);
-    }
-    debug_printf("%ld: End\n", cbuf2_write_index(cbuf2));
+    if (rmi->http_info.content_type != CONTENT_TYPE_OGG) {
+	debug_printf ("---CBUF_DEBUG_META_LIST---\n");
+	debug_printf("%ld: <size>\n", cbuf2->size);
+	debug_printf("%ld: Start\n", cbuf2->base_idx);
+	list_for_each (p, &(cbuf2->metadata_list)) {
+	    METADATA_LIST *meta_tmp;
+	    int metadata_pos;
+	    meta_tmp = list_entry (p, METADATA_LIST, m_list);
+	    metadata_pos = (meta_tmp->m_chunk + 1) * cbuf2->chunk_size - 1;
+	    debug_printf("%ld: %d [%d]%s\n", 
+			 metadata_pos,
+			 meta_tmp->m_chunk, 
+			 (int) meta_tmp->m_composed_metadata[0],
+			 &meta_tmp->m_composed_metadata[1]);
+	}
+	debug_printf("%ld: End\n", cbuf2_write_index(cbuf2));
 
-    debug_printf ("---CBUF_DEBUG_OGG_LIST---\n");
-    debug_printf("%ld: <size>\n", cbuf2->size);
-    debug_printf("%ld: Start\n", cbuf2->base_idx);
-    list_for_each (p, &(cbuf2->ogg_page_list)) {
-	OGG_PAGE_LIST *ogg_tmp;
-	ogg_tmp = list_entry (p, OGG_PAGE_LIST, m_list);
-	debug_printf("ogg page = %02x [%ld, %ld, %ld]\n", 
-		     ogg_tmp->m_page_flags,
-		     ogg_tmp->m_page_start,
-		     ogg_tmp->m_page_len,
-		     cbuf2_add(cbuf2,ogg_tmp->m_page_start,
-			       ogg_tmp->m_page_len));
-    }
-    if (cbuf2->song_page) {
-	debug_printf ("%ld(%ld): Song Page\n", 
-		      cbuf2->song_page->m_page_start,
-		      cbuf2->song_page_done);
     } else {
-	debug_printf ("---: Song Page\n");
+	debug_printf ("---CBUF_DEBUG_OGG_LIST---\n");
+	debug_printf("%ld: <size>\n", cbuf2->size);
+	debug_printf("%ld: Start\n", cbuf2->base_idx);
+	list_for_each (p, &(cbuf2->ogg_page_list)) {
+	    OGG_PAGE_LIST *ogg_tmp;
+	    ogg_tmp = list_entry (p, OGG_PAGE_LIST, m_list);
+	    debug_printf("ogg page = %02x [%ld, %ld, %ld]\n", 
+			 ogg_tmp->m_page_flags,
+			 ogg_tmp->m_page_start,
+			 ogg_tmp->m_page_len,
+			 cbuf2_add(cbuf2,ogg_tmp->m_page_start,
+				   ogg_tmp->m_page_len));
+	}
+	if (cbuf2->song_page) {
+	    debug_printf ("%ld(%ld): Song Page\n", 
+			  cbuf2->song_page->m_page_start,
+			  cbuf2->song_page_done);
+	} else {
+	    debug_printf ("---: Song Page\n");
+	}
+	debug_printf ("%ld: End\n", cbuf2_write_index(cbuf2));
+	debug_printf ("---\n");
     }
-    debug_printf ("%ld: End\n", cbuf2_write_index(cbuf2));
-    debug_printf ("---\n");
 }
 
 error_code
@@ -721,6 +733,7 @@ cbuf2_offset_2 (CBUF2 *cbuf2, u_long pos)
 }
 
 /* returns pos1 - pos2 (not used) */
+#if defined (commentout)
 static u_long
 cbuf2_subtract (CBUF2 *cbuf2, u_long pos1, u_long pos2)
 {
@@ -731,6 +744,7 @@ cbuf2_subtract (CBUF2 *cbuf2, u_long pos1, u_long pos2)
 	return diff + cbuf2->item_count;
     }
 }
+#endif
 
 static u_long
 cbuf2_idx_to_chunk (CBUF2 *cbuf2, u_long idx)
@@ -768,14 +782,14 @@ cbuf2_init_relay_entry (CBUF2 *cbuf2, RELAY_LIST* ptr, u_long burst_request)
 
     threadlib_waitfor_sem (&cbuf2->cbuf_sem);
     if (burst_request >= cbuf2->item_count) {
-	ptr->m_cbuf_pos = 0;
+	ptr->m_cbuf_offset = 0;
     } else {
-	ptr->m_cbuf_pos = cbuf2->item_count - burst_request;
+	ptr->m_cbuf_offset = cbuf2->item_count - burst_request;
     }
     if (cbuf2->content_type == CONTENT_TYPE_OGG) {
 	/* Move to ogg page boundary */
 	int no_boundary_yet = 1;
-	u_long req_pos = ptr->m_cbuf_pos;
+	u_long req_pos = ptr->m_cbuf_offset;
 	p = cbuf2->ogg_page_list.next;
 	list_for_each (p, &(cbuf2->ogg_page_list)) {
 	    u_long page_offset;
@@ -783,13 +797,13 @@ cbuf2_init_relay_entry (CBUF2 *cbuf2, RELAY_LIST* ptr, u_long burst_request)
 	    page_offset = cbuf2_offset (cbuf2, tmp->m_page_start);
 	    if (page_offset < req_pos || no_boundary_yet) {
 		if (tmp->m_page_flags & OGG_PAGE_BOS) {
-		    ptr->m_cbuf_pos = page_offset;
+		    ptr->m_cbuf_offset = page_offset;
 		    ptr->m_header_buf_ptr = 0;
 		    ptr->m_header_buf_len = 0;
 		    ptr->m_header_buf_off = 0;
 		    no_boundary_yet = 0;
 		} else if (!(tmp->m_page_flags & OGG_PAGE_2)) {
-		    ptr->m_cbuf_pos = page_offset;
+		    ptr->m_cbuf_offset = page_offset;
 		    ptr->m_header_buf_ptr = tmp->m_header_buf_ptr;
 		    ptr->m_header_buf_len = tmp->m_header_buf_len;
 		    ptr->m_header_buf_off = 0;
@@ -798,7 +812,7 @@ cbuf2_init_relay_entry (CBUF2 *cbuf2, RELAY_LIST* ptr, u_long burst_request)
 	    }
 	    debug_printf ("Computing cbuf2_offset: "
 			  "req=%d pos=%d bas=%d pag=%d off=%d hbuf=%d\n",
-			  req_pos, ptr->m_cbuf_pos, 
+			  req_pos, ptr->m_cbuf_offset, 
 			  cbuf2->base_idx, tmp->m_page_start,
 			  page_offset, ptr->m_header_buf_ptr);
 	    if (!no_boundary_yet && page_offset > req_pos) {
@@ -811,11 +825,11 @@ cbuf2_init_relay_entry (CBUF2 *cbuf2, RELAY_LIST* ptr, u_long burst_request)
 	}
     } else {
 	/* Move to metadata boundary */
-	ptr->m_cbuf_pos = (ptr->m_cbuf_pos / cbuf2->chunk_size) 
+	ptr->m_cbuf_offset = (ptr->m_cbuf_offset / cbuf2->chunk_size) 
 		* cbuf2->chunk_size;
     }
     debug_printf ("cbuf2_init_relay_entry: %d %d %d %d\n",
-		  ptr->m_cbuf_pos, 
+		  ptr->m_cbuf_offset, 
 		  cbuf2->base_idx, 
 		  cbuf2->item_count, 
 		  cbuf2->size
