@@ -181,6 +181,101 @@ sr_strncpy (char* dst, char* src, int n)
 /*****************************************************************************
  * These functions ARE mchar related
  *****************************************************************************/
+/* Convert a string, replacing unconvertable characters.
+   Returns a gchar* string, which must be freed by caller using g_free. */
+gchar*
+convert_string_with_replacement (char* instring, gsize len, 
+				 char* from_codeset, char* to_codeset,
+				 char* repl)
+{
+    GIConv giconv;
+    gsize cur = 0;              /* Current byte to convert */
+    gchar* output_string = 0;
+    GError *error = 0;
+    gsize bytes_to_convert = len;
+    int need_repl = 1;
+
+    giconv = g_iconv_open (to_codeset, from_codeset);
+    if (giconv == (GIConv)-1) {
+	/* Not sure why this would happen */
+	return output_string;
+    }
+    while (cur < len) {
+	gchar* os;
+	gsize br, bw;
+
+	br = 0;
+	os = g_convert_with_iconv (&instring[cur], bytes_to_convert, 
+				   giconv, &br, &bw, &error);
+
+	/* If the conversion was unsuccessful, usually it means that 
+	   either the input byte doesn't belong to the from_codeset, 
+	   or the code point doesn't belong to the to_codeset.
+	*/
+	if (error) {
+	    switch (error->code) {
+	    case G_CONVERT_ERROR_ILLEGAL_SEQUENCE:
+	    case G_CONVERT_ERROR_FAILED:
+	    case G_CONVERT_ERROR_PARTIAL_INPUT:
+		break;
+	    case G_CONVERT_ERROR_NO_CONVERSION:
+	    default:
+		/* This shouldn't happen, as GNU inconv guarantees 
+		   conversion to/from UTF-8. */
+		return output_string;
+	    }
+	    /* How many bytes to try next time? */
+	    switch (bytes_to_convert) {
+	    case 4:
+	    case 3:
+	    case 2:
+		bytes_to_convert --;
+		break;
+	    case 1:
+		/* Crapped out.  Drop current byte, and add "?" to string. */
+		cur ++;
+		bytes_to_convert = len - cur;
+		if (need_repl) {
+		    need_repl = 0;
+		    if (output_string) {
+			gchar *tmp;
+			tmp = g_strconcat (output_string, repl, NULL);
+			g_free (output_string);
+			output_string = tmp;
+		    } else {
+			output_string = g_strdup (repl);
+		    }
+		}
+		break;
+	    default:
+		/* Best guess based on br value returned from iconv */
+		if (br < bytes_to_convert && br > 0) {
+		    bytes_to_convert = br;
+		} else {
+		    bytes_to_convert = 4;
+		}
+	    }
+	    g_error_free (error);
+	    error = 0;
+	} else {
+	    /* Successful conversion. */
+	    need_repl = 1;
+	    if (output_string) {
+		gchar* tmp = g_strconcat (output_string, os, NULL);
+		g_free (output_string);
+		g_free (os);
+		output_string = tmp;
+	    } else {
+		output_string = os;
+	    }
+	    cur += br;
+	    bytes_to_convert = len - cur;
+	}
+    }
+    g_iconv_close (giconv);
+    return output_string;
+}
+
 #if HAVE_WCHAR_SUPPORT
 # if HAVE_ICONV
 int 
@@ -302,19 +397,6 @@ wchar_from_char (char c, const char* codeset)
 }
 #endif /* HAVE_WCHAR_SUPPORT */
 
-/* From advice of Behdad Esfahbod" <behdad@behdad.org> 
-Very good point.  I raised this here too:
-  http://bugzilla.gnome.org/show_bug.cgi?id=391261#c9
-In Pango these days I loop over the string, calling g_utf8_validate()
-and replacing any invalid bytes with -1.  The -1 byte is known to be
-safe when passed to various glib UTF-8 functions.
-*/
-#if defined (commentout)
-int
-utf8validate (gchar* s)
-{
-}
-#endif
 
 /* Assumes src is valid utf8 */
 int
@@ -358,12 +440,9 @@ mstring_from_string (RIP_MANAGER_INFO* rmi, mchar* m, int mlen,
     *m = 0;
     if (!c) return 0;
 
-#if USE_GLIB_UTF8
     {
-	GError *error = NULL;
 	gchar* mstring;
 	char* src_codeset;
-	gsize tmp;
 	int rc;
 	
 	switch (codeset_type) {
@@ -389,45 +468,18 @@ mstring_from_string (RIP_MANAGER_INFO* rmi, mchar* m, int mlen,
 	    printf ("Program error.  Bad codeset m->c (%d)\n", codeset_type);
 	    exit (-1);
 	}
-	mstring = g_convert_with_fallback 
-		(c, -1, "UTF-8", src_codeset, "?", 0, &tmp, &error);
-	if (error) {
+	/* This is the new method */
+	mstring = convert_string_with_replacement (c, strlen(c), 
+						   src_codeset, "UTF-8", 
+						   "?");
+	if (!mstring) {
 	    debug_printf ("Error converting mstring_from_string\n");
-	    g_free (mstring);
 	    return 0;
 	}
 	rc = utf8cpy (m, mstring, mlen);
 	g_free (mstring);
 	return rc;
     }
-#elif HAVE_WCHAR_SUPPORT
-    switch (codeset_type) {
-    case CODESET_UTF8:
-	return wstring_from_string (m, mlen, c, "UTF-8");
-	break;
-    case CODESET_LOCALE:
-	return wstring_from_string (m, mlen, c, mchar_cs->codeset_locale);
-	break;
-    case CODESET_FILESYS:
-	return wstring_from_string (m, mlen, c, mchar_cs->codeset_filesys);
-	break;
-    case CODESET_ID3:
-	return wstring_from_string (m, mlen, c, mchar_cs->codeset_id3);
-	break;
-    case CODESET_METADATA:
-	return wstring_from_string (m, mlen, c, mchar_cs->codeset_metadata);
-	break;
-    case CODESET_RELAY:
-	return wstring_from_string (m, mlen, c, mchar_cs->codeset_relay);
-	break;
-    default:
-	printf ("Program error.  Bad codeset m->c (%d)\n", codeset_type);
-	exit (-1);
-    }
-#else
-    sr_strncpy (m, c, mlen);
-    return strlen (m);
-#endif
 }
 
 /* Replacement for mstring_from_string using dynamic allocation */
@@ -456,7 +508,6 @@ string_from_mstring (RIP_MANAGER_INFO* rmi, char* c, int clen, mchar* m, int cod
     if (clen < 0) return 0;
     *c = 0;
     if (!m) return 0;
-#if USE_GLIB_UTF8
     {
 	GError *error = NULL;
 	gchar* cstring;
@@ -500,34 +551,6 @@ string_from_mstring (RIP_MANAGER_INFO* rmi, char* c, int clen, mchar* m, int cod
 	g_free (cstring);
 	return strlen(c);
     }
-#elif defined (HAVE_WCHAR_SUPPORT)
-    switch (codeset_type) {
-    case CODESET_UTF8:
-	return string_from_wstring (c, clen, m, "UTF-8");
-	break;
-    case CODESET_LOCALE:
-	return string_from_wstring (c, clen, m, mchar_cs->codeset_locale);
-	break;
-    case CODESET_FILESYS:
-	return string_from_wstring (c, clen, m, mchar_cs->codeset_filesys);
-	break;
-    case CODESET_ID3:
-	return string_from_wstring (c, clen, m, mchar_cs->codeset_id3);
-	break;
-    case CODESET_METADATA:
-	return string_from_wstring (c, clen, m, mchar_cs->codeset_metadata);
-	break;
-    case CODESET_RELAY:
-	return string_from_wstring (c, clen, m, mchar_cs->codeset_relay);
-	break;
-    default:
-	printf ("Program error.  Bad codeset c->m.\n");
-	exit (-1);
-    }
-#else
-    sr_strncpy (c, m, clen);
-    return strlen (c);
-#endif
 }
 
 const char*
@@ -572,12 +595,17 @@ default_codeset (void)
 }
 
 void
-set_codesets_default (CODESET_OPTIONS* cs_opt)
+sr_set_locale (void)
 {
-    const char* fromcode = 0;
     setlocale (LC_ALL, "");
     setlocale (LC_CTYPE, "");
     debug_printf ("LOCALE is %s\n",setlocale(LC_ALL,NULL));
+}
+
+void
+set_codesets_default (CODESET_OPTIONS* cs_opt)
+{
+    const char* fromcode = 0;
 
     /* Set default codesets */
     fromcode = default_codeset ();
@@ -647,40 +675,19 @@ mstrncpy (mchar* dst, mchar* src, int n)
 mchar* 
 mstrdup (mchar* src)
 {
-#if USE_GLIB_UTF8
     return strdup (src);
-#elif defined HAVE_WCHAR_SUPPORT
-    /* wstrdup/wcsdup is non-standard */
-    mchar* new_string = (mchar*) malloc (sizeof(mchar)*(wcslen(src) + 1));
-    wcscpy (new_string, src);
-    return new_string;
-#else
-    return strdup (src);
-#endif
 }
 
 mchar* 
 mstrcpy (mchar* dest, const mchar* src)
 {
-#if USE_GLIB_UTF8
     return strcpy (dest, src);
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcscpy (dest, src);
-#else
-    return strcpy (dest, src);
-#endif
 }
 
 size_t
 mstrlen (mchar* s)
 {
-#if USE_GLIB_UTF8
     return strlen (s);
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcslen (s);
-#else
-    return strlen (s);
-#endif
 }
 
 /* GCS FIX: gcc can give a warning about vswprintf.  This may require 
@@ -691,14 +698,7 @@ msnprintf (mchar* dest, size_t n, const mchar* fmt, ...)
     int rc;
     va_list ap;
     va_start (ap, fmt);
-#if USE_GLIB_UTF8
     rc = vsnprintf (dest, n, fmt, ap);
-#elif defined HAVE_WCHAR_SUPPORT
-    rc = vswprintf (dest, n, fmt, ap);
-    debug_printf ("vswprintf got %d\n", rc);
-#else
-    rc = vsnprintf (dest, n, fmt, ap);
-#endif
     va_end (ap);
     return rc;
 }
@@ -706,59 +706,29 @@ msnprintf (mchar* dest, size_t n, const mchar* fmt, ...)
 mchar*
 mstrchr (const mchar* ws, mchar wc)
 {
-#if USE_GLIB_UTF8
     return g_utf8_strchr (ws, -1, g_utf8_get_char(&wc));
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcschr (ws, wc);
-#else
-    return strchr (ws, wc);
-#endif
 }
 
 mchar*
 mstrrchr (const mchar* ws, mchar wc)
 {
-#if USE_GLIB_UTF8
     return g_utf8_strrchr (ws, -1, g_utf8_get_char(&wc));
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcsrchr (ws, wc);
-#else
-    return strrchr (ws, wc);
-#endif
 }
 
 mchar*
 mstrncat (mchar* ws1, const mchar* ws2, size_t n)
 {
-#if USE_GLIB_UTF8
     return strncat (ws1, ws2, n);
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcsncat (ws1, ws2, n);
-#else
-    return strncat (ws1, ws2, n);
-#endif
 }
 
 int
 mstrcmp (const mchar* ws1, const mchar* ws2)
 {
-#if USE_GLIB_UTF8
     return strcmp (ws1, ws2);
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcscmp (ws1, ws2);
-#else
-    return strcmp (ws1, ws2);
-#endif
 }
 
 long int
 mtol (const mchar* string)
 {
-#if USE_GLIB_UTF8
     return strtol (string, 0, 0);
-#elif defined HAVE_WCHAR_SUPPORT
-    return wcstol (string, 0, 0);
-#else
-    return strtol (string, 0, 0);
-#endif
 }
