@@ -20,6 +20,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 
 #include "rip_manager.h"
 #include "mchar.h"
@@ -32,9 +34,27 @@
 #define DEFAULT_USER_AGENT ("streamripper/" SRVERSION)
 static GKeyFile *m_key_file = NULL;
 
+enum PrefsVersion {
+    PREFS_VERSION_1_63_BETA_2,
+    PREFS_VERSION_1_63_BETA_8,
+    PREFS_VERSION_1_63_4
+};
+
+static const char* prefs_version_strings[] = {
+    "",		         // None = "1.63-beta-2"
+    "1.63-beta-8",       // Change dropcount from 0 to 1
+    "1.63.4",            // Change overwrite from larger to version
+    0
+};
+
+/* Preference file versions */
+#define PREFS_VERSION_CURRENT "1.63.4"
+
+
 /******************************************************************************
  * Private function protoypes
  *****************************************************************************/
+static enum PrefsVersion string_to_prefs_version (char* str);
 static gchar* prefs_get_config_dir (void);
 static void prefs_get_global_defaults (GLOBAL_PREFS* global_prefs);
 
@@ -53,6 +73,10 @@ static void prefs_get_long (long *dest, char *group, char *key);
 static void prefs_set_string (char* group, char* key, char* value);
 static void prefs_set_integer (char* group, char* key, gint value);
 
+/******************************************************************************
+ * Private Vars
+ *****************************************************************************/
+
 /******************************************************************************
  * Public functions
  *****************************************************************************/
@@ -73,7 +97,7 @@ prefs_load (void)
     prefs_fn = g_build_filename (prefs_dir,
 				 "streamripper.ini",
 				 NULL);
-    debug_printf ("prefs_fn = %s\n", prefs_fn);
+    debug_printf ("prefs_fn [utf8] = %s\n", prefs_fn);
 
     if (!m_key_file) {
 	m_key_file = g_key_file_new ();
@@ -86,13 +110,34 @@ prefs_load (void)
     prefs_get_global_prefs (&global_prefs);
     prefs_set_global_prefs (&global_prefs);
 
-    /* If there is no version string, then the version is "1.63-beta-2". */
-    if (rc && !global_prefs.version[0]) {
-	/* Silently update dropcount */
+    /* Silently update preferences on version change */
+    if (rc) {
 	u_long dropcount;
-	rc = prefs_get_ulong (&dropcount, "stream defaults", "dropcount");
-	if (dropcount == 0) {
-	    prefs_set_integer ("stream defaults", "dropcount", 1);
+	char overwrite_str[128];
+
+	debug_printf ("Updating prefs %d\n", 
+		      string_to_prefs_version (global_prefs.version));
+	switch (string_to_prefs_version (global_prefs.version)) {
+	case PREFS_VERSION_1_63_BETA_2:
+	    /* Silently update dropcount */
+	    rc = prefs_get_ulong (&dropcount, "stream defaults", "dropcount");
+	    if (!rc || dropcount == 0) {
+		prefs_set_integer ("stream defaults", "dropcount", 1);
+	    }
+	    /* Fall through */
+	case PREFS_VERSION_1_63_BETA_8:
+	    /* Silently update overwrite behavior */
+	    rc = prefs_get_string (overwrite_str, 128, "stream defaults",
+				   "over_write_complete");
+	    if (!rc || !strcmp (overwrite_str, "larger")) {
+		prefs_set_string ("stream defaults", "over_write_complete", 
+				  "version");
+	    }
+	    /* Fall through */
+	case PREFS_VERSION_1_63_4:
+	default:
+	    /* Prefs version is up to date -- do nothing */
+	    break;
 	}
     }
 
@@ -140,7 +185,9 @@ prefs_save (void)
     }
 
     /* Write to file */
-    fp = fopen (prefs_fn, "w");
+    /* RMK: We use glib string encoding for conversion of the filename
+	rather than --codeset-filesys here. */
+    fp = g_fopen (prefs_fn, "w");
     if (fp) {
 	fwrite (keyfile_contents, 1, keyfile_contents_len, fp);
 	fclose (fp);
@@ -344,6 +391,23 @@ debug_stream_prefs (STREAM_PREFS* prefs)
 /******************************************************************************
  * Private functions
  *****************************************************************************/
+static enum PrefsVersion
+string_to_prefs_version (char* str)
+{
+    int i = 0;
+
+    /* If there is no version string, then the version is "1.63-beta-2". */
+    if (!str) return 0;
+
+    while (prefs_version_strings[i]) {
+	if (strcmp(str, prefs_version_strings[i]) == 0) {
+	    return i;
+	}
+	i++;
+    }
+    return 0;
+}
+
 /* Calling routine must free returned string */
 static gchar* 
 prefs_get_config_dir (void)
@@ -378,7 +442,7 @@ prefs_get_stream_defaults (STREAM_PREFS* prefs)
     prefs->max_connections = 1;
     prefs->maxMB_rip_size = 0;
     prefs->flags = OPT_AUTO_RECONNECT | 
-	    OPT_SEPERATE_DIRS | 
+	    OPT_SEPARATE_DIRS | 
 	    OPT_SEARCH_PORTS |
 	    /* OPT_ADD_ID3V1 | -- removed starting 1.62-beta-2 */
 	    OPT_ADD_ID3V2 |
@@ -406,7 +470,7 @@ prefs_get_stream_defaults (STREAM_PREFS* prefs)
     set_codesets_default (&prefs->cs_opt);
 
     prefs->count_start = 0;
-    prefs->overwrite = OVERWRITE_LARGER;
+    prefs->overwrite = OVERWRITE_VERSION;
     prefs->ext_cmd[0] = 0;
 }
 
@@ -529,9 +593,8 @@ prefs_set_integer (char* group, char* key, gint value)
 static void
 prefs_get_stream_prefs_keyfile (STREAM_PREFS* prefs, char* group)
 {
-//    char* group = 0;
-    char overwrite_str[128];
     u_long temp;
+    char overwrite_str[128];
 
     if (!m_key_file) return;
 
@@ -563,7 +626,7 @@ prefs_get_stream_prefs_keyfile (STREAM_PREFS* prefs, char* group)
 	OPT_FLAG_SET (prefs->flags, OPT_AUTO_RECONNECT, temp);
     }
     if (prefs_get_ulong (&temp, group, "separate_dirs")) {
-	OPT_FLAG_SET (prefs->flags, OPT_SEPERATE_DIRS, temp);
+	OPT_FLAG_SET (prefs->flags, OPT_SEPARATE_DIRS, temp);
     }
     if (prefs_get_ulong (&temp, group, "make_relay")) {
 	OPT_FLAG_SET (prefs->flags, OPT_MAKE_RELAY, temp);
@@ -681,7 +744,7 @@ prefs_set_stream_prefs_keyfile (STREAM_PREFS* prefs, STREAM_PREFS* gp,
     prefs_set_integer (group, "auto_reconnect",
 		       OPT_FLAG_ISSET (prefs->flags, OPT_AUTO_RECONNECT));
     prefs_set_integer (group, "separate_dirs",
-		       OPT_FLAG_ISSET (prefs->flags, OPT_SEPERATE_DIRS));
+		       OPT_FLAG_ISSET (prefs->flags, OPT_SEPARATE_DIRS));
     prefs_set_integer (group, "make_relay",
 		       OPT_FLAG_ISSET (prefs->flags, OPT_MAKE_RELAY));
     prefs_set_integer (group, "add_numeric_prefix",
