@@ -56,33 +56,14 @@
 #define errno WSAGetLastError()
 #endif
 
-/*****************************************************************************
- * Global variables
- *****************************************************************************/
-RELAY_LIST* g_relay_list = NULL;
-unsigned long g_relay_list_len = 0;
-HSEM g_relay_list_sem;
-
-/*****************************************************************************
- * Private vars
- *****************************************************************************/
-static HSEM m_sem_not_connected;
-static char m_http_header[MAX_HEADER_LEN];
-static SOCKET m_listensock = SOCKET_ERROR;
-static BOOL m_running = FALSE;
-static BOOL m_initdone = FALSE;
-static THREAD_HANDLE m_hthread;
-static THREAD_HANDLE m_hthread2;
-
-static int m_max_connections;
-static int m_have_metadata;
-
 /*****************************************************************************
  * Private functions
  *****************************************************************************/
-static void thread_accept(void *notused);
-static error_code try_port(u_short port, char *if_name, char *relay_ip);
-static void thread_send(void *notused);
+static void thread_accept (void *arg);
+static error_code try_port (RELAYLIB_INFO* rli, u_short port, 
+			    char *if_name, char *relay_ip);
+static void thread_send (void *arg);
+static error_code relaylib_start_threads (RIP_MANAGER_INFO* rmi);
 
 #define BUFSIZE (1024)
 
@@ -91,21 +72,24 @@ static void thread_send(void *notused);
 #define ICY_METADATA_TAG "Icy-MetaData:"
 
 static void
-destroy_all_hostsocks(void)
+destroy_all_hostsocks (RIP_MANAGER_INFO* rmi)
 {
     RELAY_LIST* ptr;
 
-    threadlib_waitfor_sem (&g_relay_list_sem);
-    while (g_relay_list != NULL) {
-        ptr = g_relay_list;
+    threadlib_waitfor_sem (&rmi->relay_list_sem);
+    while (rmi->relay_list != NULL) {
+        ptr = rmi->relay_list;
         closesocket(ptr->m_sock);
-        g_relay_list = ptr->m_next;
-        if (ptr->m_buffer != NULL)
+        rmi->relay_list = ptr->m_next;
+        if (ptr->m_buffer != NULL) {
             free (ptr->m_buffer);
+            ptr->m_buffer = NULL;
+        }
         free(ptr);
     }
-    g_relay_list_len = 0;
-    threadlib_signal_sem (&g_relay_list_sem);
+    rmi->relay_list_len = 0;
+    rmi->relay_list = NULL;
+    threadlib_signal_sem (&rmi->relay_list_sem);
 }
 
 static int
@@ -159,9 +143,12 @@ header_receive (int sock, int *icy_metadata)
 	md = strtok (buf, HTTP_HEADER_DELIM);
 	while (md) {
 	    debug_printf ("Got token: %s\n", md);
+
 	    // Finished when we are at end of header: only CRLF will be there.
-	    if ((md[0] == '\r') && (md[1] == 0))
+	    if ((md[0] == '\r') && (md[1] == 0)) {
+		debug_printf ("End of header\n");
 		return 0;
+	    }
 	
 	    // Check for desired tag
 	    if (tag_compare (md, ICY_METADATA_TAG) == 0) {
@@ -238,11 +225,7 @@ make_nonblocking (int sock)
 #endif
 }
 
-BOOL relaylib_isrunning()
-{
-    return m_running;
-}
-
+#if defined (commentout)
 error_code
 relaylib_set_response_header(char *http_header)
 {
@@ -253,6 +236,7 @@ relaylib_set_response_header(char *http_header)
 
     return SR_SUCCESS;
 }
+#endif
 
 #ifndef WIN32
 void
@@ -265,16 +249,29 @@ catch_pipe(int code)
 #endif
 
 error_code
-relaylib_init (BOOL search_ports, u_short relay_port, u_short max_port, 
-	       u_short *port_used, char *if_name, int max_connections, 
-	       char *relay_ip, int have_metadata)
+relaylib_start (RIP_MANAGER_INFO* rmi,
+		BOOL search_ports, u_short relay_port, u_short max_port, 
+		u_short *port_used, char *if_name, 
+		int max_connections, 
+		char *relay_ip, 
+		int have_metadata)
 {
     int ret;
 #ifdef WIN32
     WSADATA wsd;
 #endif
+    RELAYLIB_INFO* rli = &rmi->relaylib_info;
 
-    debug_printf ("relaylib_init()\n");
+    /* GCS: These were globally initialized... */
+    rli->m_listensock = SOCKET_ERROR;
+    rli->m_running = FALSE;
+    rli->m_running_accept = FALSE;
+    rli->m_running_send = FALSE;
+
+    debug_printf ("relaylib_start()\n");
+
+    rmi->relay_list = 0;
+    rmi->relay_list_len = 0;
 
 #ifdef WIN32
     if (WSAStartup(MAKEWORD(2,2), &wsd) != 0) {
@@ -293,31 +290,34 @@ relaylib_init (BOOL search_ports, u_short relay_port, u_short max_port,
     signal(SIGPIPE, catch_pipe);
 #endif
 
-    if (m_initdone != TRUE) {
-        m_sem_not_connected = threadlib_create_sem();
-        g_relay_list_sem = threadlib_create_sem();
-        threadlib_signal_sem(&g_relay_list_sem);
+    rli->m_sem_not_connected = threadlib_create_sem();
+    rmi->relay_list_sem = threadlib_create_sem();
+    threadlib_signal_sem (&rmi->relay_list_sem);
 
-        // NOTE: we need to signal it here in case we try to destroy
-        // relaylib before the thread starts!
-        threadlib_signal_sem(&m_sem_not_connected);
-        m_initdone = TRUE;
-    }
-    m_max_connections = max_connections;
-    m_have_metadata = have_metadata;
+    // NOTE: we need to signal it here in case we try to destroy
+    // relaylib before the thread starts!
+    threadlib_signal_sem (&rli->m_sem_not_connected);
+
+    //m_max_connections = max_connections;
+    //m_have_metadata = have_metadata;
     *port_used = 0;
     if (!search_ports)
         max_port = relay_port;
 
     for(;relay_port <= max_port; relay_port++) {
-        ret = try_port ((u_short)relay_port, if_name, relay_ip);
+        ret = try_port (rli, (u_short)relay_port, if_name, relay_ip);
         if (ret == SR_ERROR_CANT_BIND_ON_PORT)
             continue;           // Keep searching.
 
         if (ret == SR_SUCCESS) {
             *port_used = relay_port;
             debug_printf ("Relay: Listening on port %d\n", relay_port);
-            return SR_SUCCESS;
+            ret = SR_SUCCESS;
+
+	    if (!rli->m_running) {
+		ret = relaylib_start_threads (rmi);
+	    }
+	    return ret;
         } else {
             return ret;
         }
@@ -326,18 +326,18 @@ relaylib_init (BOOL search_ports, u_short relay_port, u_short max_port,
     return SR_ERROR_CANT_BIND_ON_PORT;
 }
 
-error_code
-try_port (u_short port, char *if_name, char *relay_ip)
+static error_code
+try_port (RELAYLIB_INFO* rli, u_short port, char *if_name, char *relay_ip)
 {
     struct hostent *he;
     struct sockaddr_in local;
 
-    m_listensock = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
-    if (m_listensock == SOCKET_ERROR) {
+    rli->m_listensock = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
+    if (rli->m_listensock == SOCKET_ERROR) {
 	debug_printf ("try_port(%d) failed socket() call\n", port);
         return SR_ERROR_SOCK_BASE;
     }
-    make_nonblocking(m_listensock);
+    make_nonblocking(rli->m_listensock);
 
     if ('\0' == *relay_ip) {
 	if (read_interface(if_name,&local.sin_addr.s_addr) != 0)
@@ -354,23 +354,23 @@ try_port (u_short port, char *if_name, char *relay_ip)
     {
         // Prevent port error when restarting quickly after a previous exit
         int opt = 1;
-        setsockopt(m_listensock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+        setsockopt(rli->m_listensock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
     }
 #endif
                         
-    if (bind(m_listensock, (struct sockaddr *)&local, sizeof(local)) == SOCKET_ERROR)
+    if (bind(rli->m_listensock, (struct sockaddr *)&local, sizeof(local)) == SOCKET_ERROR)
     {
 	debug_printf ("try_port(%d) failed bind() call\n", port);
-        closesocket(m_listensock);
-        m_listensock = SOCKET_ERROR;
+        closesocket(rli->m_listensock);
+        rli->m_listensock = SOCKET_ERROR;
         return SR_ERROR_CANT_BIND_ON_PORT;
     }
         
-    if (listen(m_listensock, 1) == SOCKET_ERROR)
+    if (listen(rli->m_listensock, 1) == SOCKET_ERROR)
     {
 	debug_printf ("try_port(%d) failed listen() call\n", port);
-        closesocket(m_listensock);
-        m_listensock = SOCKET_ERROR;
+        closesocket(rli->m_listensock);
+        rli->m_listensock = SOCKET_ERROR;
         return SR_ERROR_SOCK_BASE;
     }
 
@@ -379,61 +379,90 @@ try_port (u_short port, char *if_name, char *relay_ip)
 }
 
 void
-relaylib_shutdown ()
+relaylib_stop (RIP_MANAGER_INFO* rmi)
 {
-    debug_printf("relaylib_shutdown:start\n");
-    if (!relaylib_isrunning()) {
-        debug_printf("***relaylib_shutdown:return\n");
+    int ix;
+    RELAYLIB_INFO* rli = &rmi->relaylib_info;
+
+    debug_printf("relaylib_stop:start\n");
+    if (!rli->m_running) {
+        debug_printf("***relaylib_stop:return\n");
         return;
     }
-    m_running = FALSE;
-    threadlib_signal_sem(&m_sem_not_connected);
-    if (closesocket(m_listensock) == SOCKET_ERROR) {   
+    rli->m_running = FALSE;
+    ix = 0;
+    while (ix<120 && (rli->m_running_accept | rli->m_running_send)) {
+        sleep(1);
+        ++ix;
+    }
+    if (ix==120) {
+        debug_printf ("relay threads refused to end, aborting\n");
+        fprintf (stderr,"relay threads refused to end, aborting\n");
+        exit(1);
+    }
+
+    threadlib_signal_sem (&rli->m_sem_not_connected);
+    if (closesocket(rli->m_listensock) == SOCKET_ERROR) {   
         // JCBUG, what can we do?
     }
-    m_listensock = SOCKET_ERROR;                // Accept thread will watch for this and not try to accept anymore
-    memset(m_http_header, 0, MAX_HEADER_LEN);
-    debug_printf("waiting for relay close\n");
-    threadlib_waitforclose(&m_hthread);
-    threadlib_waitforclose(&m_hthread2);
-    destroy_all_hostsocks();
-    threadlib_destroy_sem(&m_sem_not_connected);
-    m_initdone = FALSE;
+    /* Accept thread will watch for this and not try to accept anymore */
+    rli->m_listensock = SOCKET_ERROR;
 
-    debug_printf("relaylib_shutdown:done!\n");
+    debug_printf ("waiting for relay close\n");
+    threadlib_waitforclose (&rli->m_hthread_accept);
+    threadlib_waitforclose (&rli->m_hthread_send);
+    destroy_all_hostsocks (rmi);
+    threadlib_destroy_sem (&rli->m_sem_not_connected);
+
+    debug_printf("relaylib_stop:done!\n");
 }
 
-error_code
-relaylib_start ()
+static error_code
+relaylib_start_threads (RIP_MANAGER_INFO* rmi)
 {
     int ret;
+    RELAYLIB_INFO* rli = &rmi->relaylib_info;
 
-    m_running = TRUE;
-    // Spawn on a thread so it's non-blocking
-    if ((ret = threadlib_beginthread(&m_hthread, thread_accept, 0)) != SR_SUCCESS)
-        return ret;
+    rli->m_running = TRUE;
 
-    if ((ret = threadlib_beginthread(&m_hthread2, thread_send, 0)) != SR_SUCCESS)
-        return ret;
+    ret = threadlib_beginthread (&rli->m_hthread_accept, 
+				 thread_accept, (void*) rmi);
+    if (ret != SR_SUCCESS) return ret;
+    rli->m_running_accept = TRUE;
+
+    ret = threadlib_beginthread (&rli->m_hthread_send, 
+				 thread_send, (void*) rmi);
+    if (ret != SR_SUCCESS) return ret;
+    rli->m_running_send = TRUE;
 
     return SR_SUCCESS;
 }
 
 void
-thread_accept (void *notused)
+thread_accept (void* arg)
 {
     int ret;
     int newsock;
     BOOL good;
     struct sockaddr_in client;
-    int iAddrSize = sizeof(client);
+    socklen_t iAddrSize = sizeof(client);
     RELAY_LIST* newhostsock;
     int icy_metadata;
     char* client_http_header;
+    RIP_MANAGER_INFO* rmi = (RIP_MANAGER_INFO*) arg;
+    RELAYLIB_INFO* rli = &rmi->relaylib_info;
+    STREAM_PREFS* prefs = rmi->prefs;
+    int have_metadata;
+
+    if (rmi->http_info.meta_interval == NO_META_INTERVAL) {
+	have_metadata = 0;
+    } else {
+	have_metadata = 1;
+    }
 
     debug_printf("thread_accept:start\n");
 
-    while (m_running)
+    while (rli->m_running)
     {
         fd_set fds;
         struct timeval tv;
@@ -443,50 +472,50 @@ thread_accept (void *notused)
         // when a connection gets dropped, or when streamripper shuts down
         //   this event will get signaled
 	debug_printf("thread_accept:waiting on m_sem_not_connected\n");
-        threadlib_waitfor_sem (&m_sem_not_connected);
-        if (!m_running) {
+        threadlib_waitfor_sem (&rli->m_sem_not_connected);
+        if (!rli->m_running) {
 	    debug_printf("thread_accept:exit (no longer m_running)\n");
             break;
 	}
 
         // Poll once per second, instead of blocking forever in 
-	// accept(), so that we can regain control if relaylib_shutdown() 
+	// accept(), so that we can regain control if relaylib_stop() 
 	// called
         FD_ZERO (&fds);
-        while (m_listensock != SOCKET_ERROR)
+        while (rli->m_listensock != SOCKET_ERROR && rli->m_running)
         {
-            FD_SET (m_listensock, &fds);
+            FD_SET (rli->m_listensock, &fds);
             tv.tv_sec = 1;
             tv.tv_usec = 0;
 	    debug_printf("thread_accept:calling select()\n");
-            ret = select (m_listensock + 1, &fds, NULL, NULL, &tv);
+            ret = select (rli->m_listensock + 1, &fds, NULL, NULL, &tv);
 	    debug_printf("thread_accept:select() returned\n");
             if (ret == 1) {
                 unsigned long num_connected;
                 /* If connections are full, do nothing.  Note that 
                     m_max_connections is 0 for infinite connections allowed. */
-                threadlib_waitfor_sem (&g_relay_list_sem);
-                num_connected = g_relay_list_len;
-                threadlib_signal_sem (&g_relay_list_sem);
-                if (m_max_connections > 0 && num_connected >= (unsigned long) m_max_connections) {
+                threadlib_waitfor_sem (&rmi->relay_list_sem);
+                num_connected = rmi->relay_list_len;
+                threadlib_signal_sem (&rmi->relay_list_sem);
+                if (prefs->max_connections > 0 && num_connected >= (unsigned long) prefs->max_connections) {
                     continue;
                 }
                 /* Check for connections */
 		debug_printf ("Calling accept()\n");
-                newsock = accept (m_listensock, (struct sockaddr *)&client, &iAddrSize);
+                newsock = accept (rli->m_listensock, (struct sockaddr *)&client, &iAddrSize);
                 if (newsock != SOCKET_ERROR) {
                     // Got successful accept
 
-                    debug_printf ("Relay: Client %d new from %s:%hd\n", newsock,
+                    debug_printf ("Relay: Client %d new from %s:%hu\n", newsock,
                                   inet_ntoa(client.sin_addr), ntohs(client.sin_port));
 
                     // Socket is new and its buffer had better have 
 		    // room to hold the entire HTTP header!
                     good = FALSE;
-                    if (header_receive (newsock, &icy_metadata) == 0) {
+                    if (header_receive (newsock, &icy_metadata) == 0 && rmi->cbuf2.buf != NULL) {
 			int header_len;
 			make_nonblocking (newsock);
-			client_http_header = client_relay_header_generate(icy_metadata);
+			client_http_header = client_relay_header_generate (rmi, icy_metadata);
 			header_len = strlen (client_http_header);
 			ret = send (newsock, client_http_header, strlen(client_http_header), 0);
 			debug_printf ("Relay: Sent response header to client %d (%d)\n", 
@@ -496,19 +525,19 @@ thread_accept (void *notused)
                             newhostsock = malloc (sizeof(RELAY_LIST));
                             if (newhostsock != NULL) {
                                 // Add new client to list (headfirst)
-                                threadlib_waitfor_sem (&g_relay_list_sem);
+                                threadlib_waitfor_sem (&rmi->relay_list_sem);
                                 newhostsock->m_is_new = 1;
                                 newhostsock->m_sock = newsock;
-                                newhostsock->m_next = g_relay_list;
-				if (m_have_metadata) {
+                                newhostsock->m_next = rmi->relay_list;
+				if (have_metadata) {
                                     newhostsock->m_icy_metadata = icy_metadata;
 				} else {
                                     newhostsock->m_icy_metadata = 0;
 				}
 
-                                g_relay_list = newhostsock;
-                                g_relay_list_len++;
-                                threadlib_signal_sem (&g_relay_list_sem);
+                                rmi->relay_list = newhostsock;
+                                rmi->relay_list_len++;
+                                threadlib_signal_sem (&rmi->relay_list_sem);
                                 good = TRUE;
                             }
                         }
@@ -517,25 +546,29 @@ thread_accept (void *notused)
                     if (!good)
                     {
                         closesocket (newsock);
-                        debug_printf ("Relay: Client %d disconnected (Unable to receive HTTP header)\n", newsock);
+                        debug_printf ("Relay: Client %d disconnected (Unable to receive HTTP header) or cbuf2.buf is NULL\n", newsock);
+			//if (rmi->cbuf2.buf == NULL) {
+			  //  debug_printf ("In fact, cbuf2.buf is NULL\n");
+			//}
                     }
                 }
             }
             else if (ret == SOCKET_ERROR)
             {
-                // Something went wrong with select
+                /* Something went wrong with select */
                 break;
             }
         }
-        threadlib_signal_sem (&m_sem_not_connected);     // go back to accept
+        threadlib_signal_sem (&rli->m_sem_not_connected);
+	/* loop back up to select */
     }
-
-    m_running = FALSE;
+    rli->m_running_accept = FALSE;
+    rli->m_running = FALSE;
 }
 
 /* Sock is ready to receive, so send it from cbuf to relay */
 static BOOL 
-send_to_relay (RELAY_LIST* ptr)
+send_to_relay (RIP_MANAGER_INFO* rmi, RELAY_LIST* ptr)
 {
     int ret;
     int err_errno;
@@ -550,7 +583,7 @@ send_to_relay (RELAY_LIST* ptr)
 	ptr->m_offset = 0;
 	ptr->m_left_to_send = 0;
 
-	cbuf2_init_relay_entry (&g_cbuf2, ptr, burst_amount);
+	cbuf2_init_relay_entry (&rmi->cbuf2, ptr, burst_amount);
 	
 	ptr->m_is_new = 0;
     }
@@ -560,7 +593,7 @@ send_to_relay (RELAY_LIST* ptr)
 	if (!ptr->m_left_to_send) {
 	    error_code rc;
 	    ptr->m_offset = 0;
-	    rc = cbuf2_extract_relay (&g_cbuf2, ptr);
+	    rc = cbuf2_extract_relay (&rmi->cbuf2, ptr);
 	    
 	    if (rc == SR_ERROR_BUFFER_EMPTY) {
 		break;
@@ -607,7 +640,7 @@ send_to_relay (RELAY_LIST* ptr)
 }
 
 void 
-relaylib_disconnect (RELAY_LIST* prev, RELAY_LIST* ptr)
+relaylib_disconnect (RIP_MANAGER_INFO* rmi, RELAY_LIST* prev, RELAY_LIST* ptr)
 {
     RELAY_LIST* next = ptr->m_next;
     int sock = ptr->m_sock;
@@ -619,16 +652,18 @@ relaylib_disconnect (RELAY_LIST* prev, RELAY_LIST* ptr)
     if (prev != NULL) {
 	prev->m_next = next;
     } else {
-	g_relay_list = next;
+	rmi->relay_list = next;
     }
-    if (ptr->m_buffer != NULL)
+    if (ptr->m_buffer != NULL) {
 	free (ptr->m_buffer);
+        ptr->m_buffer = NULL;
+    }
     free (ptr);
-    g_relay_list_len --;
+    rmi->relay_list_len --;
 }
 
 void 
-thread_send (void *notused)
+thread_send (void* arg)
 {
     RELAY_LIST* prev;
     RELAY_LIST* ptr;
@@ -636,10 +671,12 @@ thread_send (void *notused)
     int sock;
     BOOL good;
     error_code err = SR_SUCCESS;
+    RIP_MANAGER_INFO* rmi = (RIP_MANAGER_INFO*) arg;
+    RELAYLIB_INFO* rli = &rmi->relaylib_info;
 
-    while (m_running) {
-	threadlib_waitfor_sem (&g_relay_list_sem);
-	ptr = g_relay_list;
+    while (rli->m_running) {
+	threadlib_waitfor_sem (&rmi->relay_list_sem);
+	ptr = rmi->relay_list;
 	if (ptr != NULL) {
 	    prev = NULL;
 	    while (ptr != NULL) {
@@ -649,13 +686,13 @@ thread_send (void *notused)
 		if (swallow_receive(sock) != 0) {
 		    good = FALSE;
 		} else {
-		    good = send_to_relay (ptr);
+		    good = send_to_relay (rmi, ptr);
 		}
 	       
 		if (!good) {
 		    debug_printf ("Relay: Client %d disconnected (%s)\n", 
 				  sock, strerror(errno));
-		    relaylib_disconnect (prev, ptr);
+		    relaylib_disconnect (rmi, prev, ptr);
 		} else if (ptr != NULL) {
 		    prev = ptr;
 		}
@@ -664,7 +701,8 @@ thread_send (void *notused)
 	} else {
 	    err = SR_ERROR_HOST_NOT_CONNECTED;
 	}
-	threadlib_signal_sem (&g_relay_list_sem);
+	threadlib_signal_sem (&rmi->relay_list_sem);
 	Sleep (50);
     }
+    rli->m_running_send = FALSE;
 }
