@@ -70,7 +70,8 @@ static const char* overwrite_opt_strings[] = {
     "",		// UNKNOWN
     "always",
     "never",
-    "larger"
+    "larger",
+    "version"
 };
 
 /******************************************************************************
@@ -350,7 +351,7 @@ ripthread (void *thread_arg)
     debug_stream_prefs (rmi->prefs);
 
     /* Connect to remote server */
-    ret = start_ripping(rmi);
+    ret = start_ripping (rmi);
     if (ret != SR_SUCCESS) {
 	debug_printf ("Ripthread did start_ripping()\n");
 	threadlib_signal_sem (&rmi->started_sem);
@@ -498,7 +499,7 @@ start_ripping (RIP_MANAGER_INFO* rmi)
     STREAM_PREFS* prefs = rmi->prefs;
     error_code ret;
 
-    char *pproxy = prefs->proxyurl[0] ? prefs->proxyurl : NULL;
+    const char *pproxy = prefs->proxyurl[0] ? prefs->proxyurl : NULL;
     debug_printf ("start_ripping: checkpoint 1\n");
 
     /* If proxy URL not spec'd on command line (or plugin field), 
@@ -506,7 +507,7 @@ start_ripping (RIP_MANAGER_INFO* rmi)
     if (!pproxy) {
 	char const *env_http_proxy = getenv ("http_proxy");
 	if (env_http_proxy) {
-	    strncpy (prefs->proxyurl, env_http_proxy, MAX_URL_LEN);
+	    pproxy = env_http_proxy;
 	}
     }
 
@@ -517,6 +518,7 @@ start_ripping (RIP_MANAGER_INFO* rmi)
 			   pproxy, &rmi->http_info, 
 			   prefs->useragent, prefs->if_name);
     if (ret != SR_SUCCESS) {
+	debug_printf ("http_sc_connect() returned error\n");
 	goto RETURN_ERR;
     }
 
@@ -548,11 +550,13 @@ start_ripping (RIP_MANAGER_INFO* rmi)
 	     rmi->prefs->output_directory,
 	     rmi->prefs->output_pattern,
 	     rmi->prefs->showfile_pattern,
-	     GET_SEPERATE_DIRS(rmi->prefs->flags),
+	     GET_SEPARATE_DIRS(rmi->prefs->flags),
 	     GET_DATE_STAMP(rmi->prefs->flags),
 	     rmi->http_info.icy_name);
-    if (ret != SR_SUCCESS)
+    if (ret != SR_SUCCESS) {
+	debug_printf ("filelib_init() returned error\n");
 	goto RETURN_ERR;
+    }
 
     /* Start up external program to get metadata. */
     rmi->ep = 0;
@@ -620,7 +624,7 @@ enum OverwriteOpt
 string_to_overwrite_opt (char* str)
 {
     int i;
-    for (i = 0; i < 4; i++) {
+    for (i = 0; i < 5; i++) {
 	if (strcmp(str, overwrite_opt_strings[i]) == 0) {
 	    return i;
 	}
