@@ -41,27 +41,18 @@
 #endif
 
 #include "srtypes.h"
-#include "util.h"
 #include "socklib.h"
 #include "threadlib.h"
 #include "compat.h"
 #include "debug.h"
 
-#define DEFAULT_TIMEOUT		15
 
 #if WIN32
+#define DEFAULT_TIMEOUT		(15 * 1000)
 #define FIRST_READ_TIMEOUT	(30 * 1000)
-#elif __UNIX__
-#define FIRST_READ_TIMEOUT	30
 #endif
 
 
-/****************************************************************************
- * Private Vars 
- ****************************************************************************/
-static BOOL m_done_init = FALSE; // so we don't init the mutex twice.. arg.
-
-
 /****************************************************************************
  * Function definitions
  ****************************************************************************/
@@ -74,8 +65,10 @@ socklib_init()
     int err;
 #endif
 
+#if defined (commentout)
     if (m_done_init)
 	return SR_SUCCESS;
+#endif
 
 #if WIN32
     wVersionRequested = MAKEWORD( 2, 2 );
@@ -84,7 +77,10 @@ socklib_init()
         return SR_ERROR_WIN32_INIT_FAILURE;
 #endif
 
+#if defined (commentout)
     m_done_init = TRUE;
+#endif
+
     return SR_SUCCESS;
 }
 
@@ -131,13 +127,24 @@ socklib_open(HSOCKET *socket_handle, char *host, int port, char *if_name)
     if (!socket_handle || !host)
 	return SR_ERROR_INVALID_PARAM;
 
+    /* On error:
+       Unix returns -1 and sets errno.
+       Windows??? */
     socket_handle->s = socket(AF_INET, SOCK_STREAM, 0);
+    if (socket_handle->s == SOCKET_ERROR) {
+	debug_printf ("socket() failed\n");
+	WSACleanup();
+	return SR_ERROR_CANT_CREATE_SOCKET;
+    }
 
     if (if_name) {
 	if (read_interface(if_name,&local.sin_addr.s_addr) != 0)
 	    local.sin_addr.s_addr = htonl(INADDR_ANY);
 	local.sin_family = AF_INET;
 	local.sin_port = htons(0);
+	/* On error:
+	   Unix returns -1 and sets errno.
+	   Windows??? */
 	if (bind(socket_handle->s, (struct sockaddr *)&local, 
 		 sizeof(local)) == SOCKET_ERROR) {
 	    debug_printf ("Bind failed\n");
@@ -154,6 +161,8 @@ socklib_open(HSOCKET *socket_handle, char *host, int port, char *if_name)
 	} else {
 	    debug_printf("resolving hostname: %s failed\n", host);
 	    WSACleanup();
+	    /* GCS Added... */
+	    closesocket(socket_handle->s);
 	    return SR_ERROR_CANT_RESOLVE_HOSTNAME;
 	}
     }
@@ -164,16 +173,22 @@ socklib_open(HSOCKET *socket_handle, char *host, int port, char *if_name)
     rc = connect (socket_handle->s, (struct sockaddr *)&address, len);
     if (rc == SOCKET_ERROR) {
 	debug_printf("connect failed\n");
+	/* GCS Added... */
+	WSACleanup();
+	closesocket(socket_handle->s);
 	return SR_ERROR_CONNECT_FAILED;
     }
 
 #ifdef WIN32
     {
-	struct timeval timeout = {DEFAULT_TIMEOUT*1000, 0};
+	struct timeval timeout = {DEFAULT_TIMEOUT, 0};
 	rc = setsockopt (socket_handle->s, SOL_SOCKET,  SO_RCVTIMEO, 
 			 (char *)&timeout, sizeof(timeout));
 	if (rc == SOCKET_ERROR) {
 	    debug_printf("setsockopt failed\n");
+	    /* GCS Added... */
+	    WSACleanup();
+	    closesocket(socket_handle->s);
 	    return SR_ERROR_CANT_SET_SOCKET_OPTIONS;
 	}
     }
@@ -183,22 +198,25 @@ socklib_open(HSOCKET *socket_handle, char *host, int port, char *if_name)
     return SR_SUCCESS;
 }
 
-void socklib_cleanup()
+void
+socklib_cleanup()
 {
     WSACleanup();
+#if defined (commentout)
     m_done_init = FALSE;
+#endif
 }
 
-void socklib_close(HSOCKET *socket_handle)
+void
+socklib_close(HSOCKET *socket_handle)
 {
     closesocket(socket_handle->s);
     socket_handle->closed = TRUE;
 }
 
 error_code
-socklib_read_header(HSOCKET *socket_handle, char *buffer, int size, 
-		    int (*recvall)(HSOCKET *socket_handle, char* buffer,
-				   int size, int timeout))
+socklib_read_header(RIP_MANAGER_INFO* rmi, HSOCKET *socket_handle, 
+		    char *buffer, int size)
 {
     int i;
 #ifdef WIN32
@@ -206,15 +224,10 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
 #endif
     int ret;
     char *t;
-    int (*myrecv)(HSOCKET *socket_handle, char* buffer, int size, int timeout);
 
     if (socket_handle->closed)
 	return SR_ERROR_SOCKET_CLOSED;
 
-    if (recvall)
-	myrecv = recvall;
-    else
-	myrecv = socklib_recvall;
 #ifdef WIN32
     if (setsockopt(socket_handle->s, SOL_SOCKET,  SO_RCVTIMEO, (char *)&timeout, sizeof(timeout)) == SOCKET_ERROR)
 	return SR_ERROR_CANT_SET_SOCKET_OPTIONS;
@@ -223,8 +236,10 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
     memset(buffer, 0, size);
     for(i = 0; i < size; i++)
     {
-	if ((ret = (*myrecv)(socket_handle, &buffer[i], 1, 0)) < 0)
+	ret = socklib_recvall (rmi, socket_handle, &buffer[i], 1, 0);
+	if (ret < 0) {
 	    return ret;
+	}
 
 	if (ret == 0) {
 	    debug_printf("http header:\n%s\n", buffer);
@@ -234,9 +249,8 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
 	if (socket_handle->closed)
 	    return SR_ERROR_SOCKET_CLOSED;
 
-	/* GCS: This patch is too restrictive. */
 #if defined (commentout)
-	//look for the end of the icy-header
+	/* This is too restrictive.  Some servers do not send this. */
 	if (!strstr(buffer, "icy-") && !strstr(buffer,"ice-"))
 	    continue;
 #endif
@@ -245,8 +259,8 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
 
 	if (strncmp(t, "\r\n\r\n", 4) == 0)
 	    break;
-
-	if (strncmp(t, "\n\0\r\n", 4) == 0)	// wtf? live365 seems to do this
+	/* Allegedly live365 used to do this */
+	if (strncmp(t, "\n\0\r\n", 4) == 0)
 	    break;
     }
 
@@ -258,7 +272,7 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
     buffer[i] = '\0';
 
 #ifdef WIN32
-    timeout.tv_sec = DEFAULT_TIMEOUT*1000;
+    timeout.tv_sec = DEFAULT_TIMEOUT;
     if (setsockopt(socket_handle->s, SOL_SOCKET,  SO_RCVTIMEO, (char *)&timeout, sizeof(timeout)) == SOCKET_ERROR)
 	return SR_ERROR_CANT_SET_SOCKET_OPTIONS;
 #endif
@@ -266,8 +280,9 @@ socklib_read_header(HSOCKET *socket_handle, char *buffer, int size,
     return SR_SUCCESS;
 }
 
-int
-socklib_recvall (HSOCKET *socket_handle, char* buffer, int size, int timeout)
+error_code
+socklib_recvall (RIP_MANAGER_INFO* rmi, HSOCKET *socket_handle, 
+		 char* buffer, int size, int timeout)
 {
     int ret = 0, read = 0;
     int sock;
@@ -283,15 +298,27 @@ socklib_recvall (HSOCKET *socket_handle, char* buffer, int size, int timeout)
 	if (timeout > 0) {
 	    /* Wait up to 'timeout' seconds for data on socket to be 
 	       ready for read */
+#if __UNIX__
+	    FD_SET(rmi->abort_pipe[0], &fds);
+#endif
 	    FD_SET(sock, &fds);
 	    tv.tv_sec = timeout;
 	    tv.tv_usec = 0;
 	    ret = select(sock + 1, &fds, NULL, NULL, &tv);
-	    if (ret == SOCKET_ERROR)
+	    if (ret == SOCKET_ERROR) {
+		/* This happens when I kill winamp while ripping */
 		return SR_ERROR_SELECT_FAILED;
-	    if (ret != 1)
+	    }
+	    if (ret == 0) {
 		return SR_ERROR_TIMEOUT;
+	    }
 	}
+#if __UNIX__
+	if (FD_ISSET(rmi->abort_pipe[0], &fds)) {
+	    debug_printf ("socklib_recvall detected write to abort pipe.\n");
+	    return SR_ERROR_ABORT_PIPE_SIGNALLED;
+	}
+#endif
 
         ret = recv(socket_handle->s, &buffer[read], size, 0);
 	debug_printf ("RECV req %5d bytes, got %5d bytes\n", size, ret);
