@@ -17,13 +17,9 @@
 #ifndef __SRTYPES_H__
 #define __SRTYPES_H__
 
-#define USE_GLIB_UTF8 1
-
-#if (USE_GLIB_UTF8)
 #include <glib.h>
-#endif
-#include "regex.h"
 
+#include "regex.h"
 #include "srconfig.h"
 #include "compat.h"
 #include "list.h"
@@ -82,10 +78,6 @@ typedef unsigned int uint32_t;
 
 #define NO_META_INTERVAL	-1
 
-/* Preference file versions */
-#define PREFS_VERSION_1_63_BETA_8           "1.63-beta-8"
-#define PREFS_VERSION_CURRENT PREFS_VERSION_1_63_BETA_8
-
 /* GCS - Grr. I don't care.  Max path is 254 until I get around to
     fixing this for other platforms. */
 #define SR_MAX_PATH		254
@@ -205,13 +197,7 @@ typedef struct CODESET_OPTIONSst
 /* 
  * Wide character support
  */
-#if USE_GLIB_UTF8
 typedef gchar mchar;
-#elif HAVE_WCHAR_SUPPORT
-typedef wchar_t mchar;
-#else
-typedef char mchar;
-#endif
 
 /* 
  * Parse_Rule is a single line of the parse rules file
@@ -265,7 +251,8 @@ enum OverwriteOpt {
     OVERWRITE_UNKNOWN,	// Error case
     OVERWRITE_ALWAYS,	// Always replace file in complete with newer
     OVERWRITE_NEVER,	// Never replace file in complete with newer
-    OVERWRITE_LARGER	// Replace file in complete if newer is larger
+    OVERWRITE_LARGER,	// Replace file in complete if newer is larger
+    OVERWRITE_VERSION	// Never overwrite, instead make a new version
 };
 
 /* Information extracted from the stream's HTTP header */
@@ -585,7 +572,7 @@ struct RIP_MANAGER_INFOst
     unsigned int cue_sheet_bytes;
 
     /* Keep track of how many tracks we have ripped */
-    int track_count;
+    unsigned int track_count;
 
     /* The circular buffer */
     CBUF2 cbuf2;
